////////////////////////////////////////////////////////////////////////////////
/// @file
/// @brief シェルコマンドの実行に関する関数のソース
///
/// IPアドレスやMACアドレスの取得に利用されます。
///
/// Copyright (C) 2009 Alpha Systems Inc. All Rights Reserved.
/// @internal
/// @date $Date: $
/// @version $Revision: $
////////////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include "vncviewer.h"

#define dbg(...) (fprintf(stderr, "@DEBUG %s:%u:%s(): ", __FILE__, __LINE__, __func__),fprintf(stderr, " "__VA_ARGS__), fprintf(stderr, "\n"))


/// 外部コマンドの実行
///
/// 外部コマンドcmdを実行する。
///
/// 引数waitが0の場合は子プロセスが非同期モードで実行され、
/// 直ちに親プロセスに制御を返す。waitが1の場合は子プロセス
/// が同期モードで実行される。
///
/// また、waitが1の場合のみ引数resには子プロセスの戻り値が
/// 格納される。waitが0の場合はresは更新されない。関数の
/// 利用者は、子プロセスの戻り値を取得する必要がない場合は
/// resにNULLをセットしておく事。
///
/// @param cmd  [in]  実行されるコマンド
/// @param wait [in]  直ちに終了するかどうか
/// @param res  [out] 子プロセスの戻り値
/// @retval  0 正常終了
/// @retval -1 異常終了
///
/// @note Windows版では、CreateProcess() 関数が実行される。
/// コンソール窓の出現を抑止するために、引数のSTARTUPINFO
/// 構造体にはSW_HIDEが指定される。
///
int runCommand(const char *cmd, int wait, int *res)
{
	STARTUPINFO sinfo;
	PROCESS_INFORMATION pinfo;
	char cmdline[CMD_BUF_MAX];
	BOOL  err;

	if (cmd == NULL) {
		dbg("address is null pointer.");
		return -1;
	}

	if ((wait != 0) && (wait != 1)) {
		dbg("invalid param.");
		return -1;
	}

	// STARTUPINFO構造体(ウインドウ非表示)
	memset(&sinfo, 0, sizeof(sinfo));
	sinfo.cb          = sizeof(sinfo);
	sinfo.dwFlags     = STARTF_USESHOWWINDOW;
	sinfo.wShowWindow = SW_HIDE;

	// CreateProcess()の第2引数にはリテラルのままで渡さない
	// (Unicode版のCreateProcessW()で失敗するため)
	memset(cmdline, 0, sizeof(cmdline));
	strcpy(cmdline, "cmd.exe /c ");
	strcat(cmdline, cmd);

	// プロセスの実行
	err = CreateProcess(NULL, cmdline, NULL, NULL, FALSE,
		NORMAL_PRIORITY_CLASS, NULL, NULL, &sinfo, &pinfo);
	if (err == 0) {
		dbg("CreateProcess() failed.");
		return -1;
	}

	// 同期モードの場合
	if (wait == 1) {
		// 子プロセスの終了まで待機
		err = WaitForSingleObject(pinfo.hProcess, INFINITE);
		if (err == WAIT_FAILED) {
			dbg("WaitForSingleObject() failed.");
			CloseHandle(pinfo.hThread);
			CloseHandle(pinfo.hProcess);
			return -1;

		}

		// resがNULLでなければ、子プロセスの終了コード取得
		if (res != NULL) {
			err = GetExitCodeProcess(pinfo.hProcess, (DWORD *) res);
			if (err == 0) {
				dbg("GetExitCodeProcess() failed.");
				CloseHandle(pinfo.hThread);
				CloseHandle(pinfo.hProcess);
				return -1;
			}
		}
	}

	CloseHandle(pinfo.hThread);
	CloseHandle(pinfo.hProcess);

	return 0;
}

/// 外部コマンドの標準出力を文字列で取得
///
/// 外部コマンドcmdの標準出力outに格納する、引数outのバッファは
/// 関数の利用者が確保しておき、そのバッファ長を引数lenで渡す必
/// 要がある。
///
/// @param cmd  [in]  実行されるコマンド
/// @param out  [out] cmdの標準出力
/// @param len  [in]  outのバッファ長
/// @retval 0  正常終了
/// @retval -1 異常終了
///
/// @note Windows版では、CreateProcess() 関数が実行される。
/// コンソール窓の出現を抑止するために、引数のSTARTUPINFO
/// 構造体にはSW_HIDEが指定される。
///
/// @warning 子プロセスの戻り値が0でない場合、この関数は子プロ
/// セスが異常終了したものとして扱い、標準出力を取得しない。
///
int getCommandStdout(const char *cmd, char **out, int len)
{
	STARTUPINFO sinfo;
	PROCESS_INFORMATION pinfo;
	SECURITY_ATTRIBUTES sa;
	HANDLE hReadPipe  = NULL;
	HANDLE hWritePipe = NULL;
	char cmdline[CMD_BUF_MAX];
	BOOL  err;
	DWORD res;
	DWORD readBytes;

	if ((cmd == NULL) || (out == NULL)) {
		dbg("address is null pointer.");
		return -1;
	}

	if (*out == NULL) {
		dbg("address is null pointer.");
		return -1;
	}

	// SECURITY_ATTRIBUTES構造体(ハンドル継承指定有り)
	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.lpSecurityDescriptor = NULL;
	sa.bInheritHandle       = TRUE;

	// 標準出力バッファ用のパイプの生成
	err = CreatePipe(&hReadPipe, &hWritePipe, &sa, STDOUT_BUF_SIZE_MAX);
	if (err == 0) {
		dbg("CreatePipe() failed.");
		return -1;
	}

	// STARTUPINFO構造体(ウインドウ非表示, 標準出力ハンドル指定有り)
	memset(&sinfo, 0, sizeof(sinfo));
	sinfo.cb          = sizeof(sinfo);
	sinfo.dwFlags     = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
	sinfo.wShowWindow = SW_HIDE;
	sinfo.hStdInput   = GetStdHandle(STD_INPUT_HANDLE);
	sinfo.hStdOutput  = hWritePipe;
	sinfo.hStdError   = GetStdHandle(STD_ERROR_HANDLE);

	// CreateProcess()の第2引数にはリテラルのままで渡さない
	// (Unicode版のCreateProcessW()で失敗するため)
	memset(cmdline, 0, sizeof(cmdline));
	strcpy(cmdline, "cmd.exe /c ");
	strcat(cmdline, cmd);

	// 子プロセスの実行
	err = CreateProcess(NULL, cmdline, NULL, NULL, TRUE,
		NORMAL_PRIORITY_CLASS, NULL, NULL, &sinfo, &pinfo);
	if (err == 0) {
		dbg("CreateProcess() failed.");
		CloseHandle(hReadPipe);
		CloseHandle(hWritePipe);
		return -1;
	}

	// 子プロセスの終了まで待機
	err = WaitForSingleObject(pinfo.hProcess, INFINITE);
	if (err == WAIT_FAILED) {
		dbg("WaitForSingleObject() failed.");
		CloseHandle(pinfo.hThread);
		CloseHandle(pinfo.hProcess);
		CloseHandle(hReadPipe);
		CloseHandle(hWritePipe);
		return -1;
	}

	// 子プロセスの終了コード取得
	err = GetExitCodeProcess(pinfo.hProcess, &res);
	if (err == 0) {
		dbg("GetExitCodeProcess() failed.");
		CloseHandle(pinfo.hThread);
		CloseHandle(pinfo.hProcess);
		CloseHandle(hReadPipe);
		CloseHandle(hWritePipe);
		return -1;
	}

	// 子プロセスの終了コードが0でない場合、異常終了
	if (res != 0) {
		dbg("GetExitCodeProcess() failed.");
		CloseHandle(pinfo.hThread);
		CloseHandle(pinfo.hProcess);
		CloseHandle(hReadPipe);
		CloseHandle(hWritePipe);
		return -1;
	}

	// パイプに格納されている標準出力のバッファ長の取得
	// (readBytesには読み取り可能なバッファ長が格納される)
	err = PeekNamedPipe(hReadPipe, NULL, 0, NULL, &readBytes, NULL);
	if (err == 0) {
		dbg("PeekNamedPipe() failed.");
		CloseHandle(pinfo.hThread);
		CloseHandle(pinfo.hProcess);
		CloseHandle(hReadPipe);
		CloseHandle(hWritePipe);
		return -1;
	}

	// 読み取り可能なバッファ長がoutの確保バッファを超える場合、異常終了
	if ((readBytes <= 0) || (readBytes >= len)) {
		dbg("invaild buffer size. (readBytes=%d)", (int) readBytes);
		CloseHandle(pinfo.hThread);
		CloseHandle(pinfo.hProcess);
		CloseHandle(hReadPipe);
		CloseHandle(hWritePipe);
		return -1;
	}

	// 標準出力の取得
	// (readBytesには読み取ったバッファ長が格納されるが使用されない)
	memset(*out, 0, len);
	err = ReadFile(hReadPipe, *out, len-1, &readBytes, NULL);
	if (err == 0) {
		dbg("ReadFile() failed.");
		CloseHandle(pinfo.hThread);
		CloseHandle(pinfo.hProcess);
		CloseHandle(hReadPipe);
		CloseHandle(hWritePipe);
		return -1;
	}

	CloseHandle(pinfo.hThread);
	CloseHandle(pinfo.hProcess);
	CloseHandle(hReadPipe);
	CloseHandle(hWritePipe);

	return 0;
}
