////////////////////////////////////////////////////////////////////////////////
/// @file
/// @brief ネットワークインターフェイス関連のソース
///
/// IPアドレスやMACアドレスの取得に利用されます。
///
/// Copyright (C) 2009 Alpha Systems Inc. All Rights Reserved.
/// @internal
/// @date $Date: $
/// @version $Revision: $
////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <errno.h>
#include <time.h>

#include "vncviewer.h"

/* ネットワークインターフェイスデバイスのIPアドレスを取得 */
int get_ifaddr(char *ifname, char *addr)
{
	int err;
	IP_ADAPTER_ADDRESSES *addrs;
	IP_ADAPTER_ADDRESSES *curaddr;
	unsigned long size;
	char *ip;
	char buf[256];

	if (addr == NULL) {
		return -1;
	}

	// 1度目でサイズを取得して、2度目でアドレス情報を取得する
	size = 0;
	err = GetAdaptersAddresses(AF_INET, 0, NULL, NULL, &size);
	if (err != ERROR_BUFFER_OVERFLOW) {
		fprintf(stderr, "GetAdapterAddresses() failed.\n");
		return -1;
	}
	addrs = (IP_ADAPTER_ADDRESSES *)malloc(size);
	if (addrs == NULL) {
		fprintf(stderr, "malloc() failed.\n");
		return -1;
	}
	err = GetAdaptersAddresses(AF_INET, 0, NULL, addrs, &size);
	if (err != ERROR_SUCCESS) {
		fprintf(stderr, "GetAdapterAddresses() failed.\n");
		free(addrs);
		return -1;
	}

	curaddr = addrs;
	if (ifname != NULL) {
		while (curaddr != NULL) {
			memset(buf, 0, sizeof(buf));
			sprintf(buf, "%ls", curaddr->FriendlyName);
			if (strcmp(buf, ifname) == 0) {
				break;
			}
			curaddr = curaddr->Next;
		}
		if (curaddr == NULL) {
			fprintf(stderr, "Network device not found\n");
			free(addrs);
			return -1;
		}
	}

	ip = inet_ntoa(((struct sockaddr_in *)(curaddr->FirstUnicastAddress->Address.lpSockaddr))->sin_addr);
	memset(addr, 0, IF_LENGTH);
	strncpy(addr, ip, IF_LENGTH-1);

	free(addrs);
	return 0;
}

/* ネットワークインターフェイスデバイスのMACアドレスを取得 */
int get_macaddr(char *ifname, unsigned char *addr)
{
	int err;
	IP_ADAPTER_ADDRESSES *addrs;
	IP_ADAPTER_ADDRESSES *curaddr;
	unsigned long size;
	int i;
	char buf[256];

	if (addr == NULL) {
		return -1;
	}

	// 1度目でサイズを取得して、2度目でアドレス情報を取得する
	size = 0;
	err = GetAdaptersAddresses(AF_INET, 0, NULL, NULL, &size);
	if (err != ERROR_BUFFER_OVERFLOW) {
		fprintf(stderr, "GetAdapterAddresses() failed.\n");
		return -1;
	}
	addrs = (IP_ADAPTER_ADDRESSES *)malloc(size);
	if (addrs == NULL) {
		fprintf(stderr, "malloc() failed.\n");
		return -1;
	}
	err = GetAdaptersAddresses(AF_INET, 0, NULL, addrs, &size);
	if (err != ERROR_SUCCESS) {
		fprintf(stderr, "GetAdapterAddresses() failed.\n");
		free(addrs);
		return -1;
	}

	curaddr = addrs;
	if (ifname != NULL) {
		while (curaddr != NULL) {
			memset(buf, 0, sizeof(buf));
			sprintf(buf, "%ls", curaddr->FriendlyName);
			if (strcmp(buf, ifname) == 0) {
				break;
			}
			curaddr = curaddr->Next;
		}
		if (curaddr == NULL) {
			fprintf(stderr, "Network device not found\n");
			free(addrs);
			return -1;
		}
	}

	memset(addr, 0, MACADDR_SIZE);
	for (i = 0; i < MACADDR_SIZE; i++) {
		addr[i] = (unsigned char)curaddr->PhysicalAddress[i];
	}

	free(addrs);
	return 0;
}

/* ソケットからMACアドレスを取得 */
int find_macaddr(int sock, unsigned char *addr)
{
	struct sockaddr_in src_addr;
	socklen_t len;

  int err;
  IP_ADAPTER_ADDRESSES *addrs;
  IP_ADAPTER_ADDRESSES *curaddr;
  unsigned long size;
  int i;
  char buf[256];

	// 指定されたソケットのIPアドレスを取得
	memset(&src_addr, 0, sizeof(src_addr));
	len = sizeof(src_addr);
	if (getsockname(sock, (struct sockaddr *)&src_addr, &len)) {
		return -1;
	}

	// 1度目でサイズを取得して、2度目でアドレス情報を取得する
	size = 0;
	err = GetAdaptersAddresses(AF_INET, 0, NULL, NULL, &size);
	if (err != ERROR_BUFFER_OVERFLOW) {
		fprintf(stderr, "GetAdapterAddresses() failed.\n");
		return -1;
	}
	addrs = (IP_ADAPTER_ADDRESSES *)malloc(size);
	if (addrs == NULL) {
		fprintf(stderr, "malloc() failed.\n");
		return -1;
	}
	err = GetAdaptersAddresses(AF_INET, 0, NULL, addrs, &size);
	if (err != ERROR_SUCCESS) {
		fprintf(stderr, "GetAdapterAddresses() failed.\n");
		free(addrs);
		return -1;
	}

	curaddr = addrs;
	while (curaddr != NULL) {
		memset(buf, 0, sizeof(buf));
		sprintf(buf, "%s", inet_ntoa(((struct sockaddr_in *)(curaddr->FirstUnicastAddress->Address.lpSockaddr))->sin_addr));
		if (strcmp(buf, inet_ntoa(src_addr.sin_addr)) == 0) {
			break;
		}
		curaddr = curaddr->Next;
	}
	if (curaddr == NULL) {
		fprintf(stderr, "Network device not found\n");
		free(addrs);
		return -1;
	}

	memset(addr, 0, MACADDR_SIZE);
	for (i = 0; i < MACADDR_SIZE; i++) {
		addr[i] = (unsigned char)curaddr->PhysicalAddress[i];
	}

	return -1;
}
